import json
import logging
import time

from pathlib import Path
from django.core.mail import EmailMultiAlternatives, get_connection
from django.core.management import BaseCommand
from slack_sdk import WebClient, errors

from database_locks import locked
from notifications.models import Notification, Subscription
from django.conf import settings

logger = logging.getLogger(__name__)


@locked
class Command(BaseCommand):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__sc = WebClient(settings.NOTIFICATIONS_SLACK_APP_TOKEN)
        self.__slack_limited = time.time()

    def add_arguments(self, parser):
        parser.add_argument('-1', '--run-once', action='store_true', default=False, help='Run only one check')

    def handle_tick(self):
        for notification in Notification.objects.filter(status=Notification.STATUS_PENDING).select_related(
            'subscription'
        ):
            try:
                if notification.subscription.service == Subscription.Service.SLACK:
                    if self.__slack_limited < time.time():
                        self.__send_slack_notifications(notification)
                elif notification.subscription.service == Subscription.Service.MAIL:
                    self.__send_email_notifications(notification)
                else:
                    notification.status = Notification.STATUS_ERROR
                    logger.error(
                        'notify failed - %d - bad service %s', notification.pk, notification.subscription.service
                    )
            except Exception as e:
                notification.status = Notification.STATUS_ERROR
                notification.save()
                logger.exception(e)

    def handle(self, *args, **options):
        """
        Main method that starts the infinite loop to fetch for pending notifications.
        When those exists, then it calls its sub methods to send Slack and Email notifications according to their types.
        """
        while True:
            self.handle_tick()
            if options['run_once']:
                break
            time.sleep(1)

    def __send_slack_notifications(self, notification):
        """
        Method responsible for handling slack notifications.
        Provided a single notifications that still is in pending state and that its subscription type is slack,
        then the method sends it using the slackclient module.
        If the notification is sent successfully then it updates its Status field to to Status_SENT. Otherwise,
        the Notification's Status property is changed to STATUS_ERROR.
        :param notification: Set of notifications with SLACK subscription and PENDING status.
        """
        try:
            self.__sc.chat_postMessage(
                # text still required for message preview (in notifications)
                text=notification.message,
                channel=notification.target,
                **notification.slack_options,
            )
            notification.status = Notification.STATUS_SENT
            notification.save(update_fields=['status'])
        except errors.SlackApiError as e:
            if e.response.get('error') == 'ratelimited':
                # handle rate limit
                try:
                    retry_after = int(e.response.headers.get('retry-after')) + 5
                except (ValueError, TypeError):
                    # if no header (weird), wait 15s
                    retry_after = 15
                self.__slack_limited = time.time() + retry_after
                logger.warning('rate limited on %d - waiting %d secs', notification.pk, retry_after)
            else:
                notification.status = Notification.STATUS_ERROR
                logger.exception('notify failed - %d - %s', notification.pk, e.response.get('error'))
                notification.save(update_fields=['status'])

    def __send_email_notifications(self, notification):
        """
        Method responsible for sending email notifications.
        Provided a single notification that still is in pending state and that is subscription type mail,
        then the method sends it to the list of targets using the Django.Core.Email dependency.
        If the email is sent successfully, then the Notification's status is changed to STATUS_SENT. Otherwise,
        the Notification's status property is changed to STATUS_ERROR.
        :param notification: Single notification of MAIL subscription with PENDING status.
        """
        email_args = json.loads(notification.options)
        with get_connection() as connection:
            msg = EmailMultiAlternatives(
                subject=email_args.get('subject'),
                body=notification.message,
                from_email=email_args.get('from_email'),
                to=json.loads(notification.target),
                reply_to=email_args.get('reply_to'),
                connection=connection,
            )
            if email_args.get("attachments"):
                MEDIA_ROOT = Path(settings.MEDIA_ROOT).resolve()
                for attach in email_args.get("attachments"):
                    path = Path(attach[1]).resolve()
                    try:
                        # check if path is relative to MEDIA_ROOT
                        path.relative_to(MEDIA_ROOT)
                    except ValueError:
                        logger.error('invalid path for attachment: %s', path)
                        continue

                    if path.exists() and not path.is_dir():
                        with path.open() as attachment:
                            msg.attach(attach[0], attachment.read(), attach[2])
                    else:
                        logger.error('could not open file from path: %s', path)
            if email_args.get('html_message'):
                msg.attach_alternative(email_args.get('html_message'), 'text/html')
            msg.send()
            notification.status = Notification.STATUS_SENT
            notification.save(update_fields=['status'])



# Generated by Django 3.2.22 on 2023-10-16 11:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0003_auto_20210913_1050"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(fields=["status"], name="notificatio_status_d92267_idx"),
        ),
    ]



import json

from django.db import models
from django.template.defaultfilters import truncatechars


class Event(models.Model):
    name = models.CharField(max_length=50, primary_key=True)
    external_token = models.CharField(
        max_length=50, null=True, blank=True, help_text="If set, notifications be posted to this event through the API"
    )
    slack_username = models.CharField(
        max_length=50, null=True, blank=True, help_text='Choose a display name for Slack bot (instead of default)'
    )
    slack_icon = models.CharField(
        max_length=50, null=True, blank=True, help_text='Choose an icon for Slack bot (instead of default)'
    )
    slack_unfurl_links = models.BooleanField(
        default=True, help_text='Pass true to enable unfurling of primarily text-based content on Slack'
    )
    mail_from = models.CharField(
        max_length=200, null=True, blank=True, help_text='Choose the sender address of the email (instead of default)'
    )
    mail_reply_to = models.CharField(
        max_length=200, null=True, blank=True, help_text='Choose the reply-to address of the email (instead of none)'
    )

    def __str__(self) -> str:
        return self.name

    def slack_api_kwargs(self):
        api_kwargs = {}
        if not self.slack_unfurl_links:
            api_kwargs['unfurl_links'] = 0
        if self.slack_username:
            api_kwargs['username'] = self.slack_username
            api_kwargs['as_user'] = 0
        else:
            api_kwargs['as_user'] = 1
        if self.slack_icon:
            api_kwargs['icon_emoji'] = self.slack_icon
        return api_kwargs

    class Meta:
        verbose_name = 'Event'
        verbose_name_plural = 'Events'


class Subscription(models.Model):
    class Service(models.TextChoices):
        SLACK = 'S'
        MAIL = 'M'

    event = models.ForeignKey('Event', null=True, on_delete=models.CASCADE)
    service = models.CharField(max_length=1, choices=Service.choices)
    target = models.TextField(blank=True, help_text='You can specify multiple targets, one per line.')
    enabled = models.BooleanField(default=True)

    def __str__(self):
        return f'[{self.event}] {self.get_service_display()}: {truncatechars(self.target, 10)}'

    class Meta:
        verbose_name = 'Subscription'
        verbose_name_plural = 'Subscriptions'


class Notification(models.Model):
    STATUS_ERROR = -1
    STATUS_PENDING = 0
    STATUS_SENT = 1

    STATUS_TYPES = (
        (STATUS_PENDING, 'Pending'),
        (STATUS_SENT, 'Sent'),
        (STATUS_ERROR, 'Error'),
    )
    time = models.DateTimeField(auto_now_add=True)
    subscription = models.ForeignKey('notifications.Subscription', null=True, on_delete=models.deletion.SET_NULL)
    message = models.TextField()
    status = models.IntegerField(default=0, choices=STATUS_TYPES)
    target = models.TextField(null=True, default=None)
    options = models.TextField(null=True, default=None)

    def __str__(self) -> str:
        return f'[{self.get_status_display()}] {self.subscription}'

    @property
    def options_dict(self):
        if self.options:
            return json.loads(self.options)
        return {}

    @property
    def slack_options(self):
        options = self.options_dict
        if 'blocks' not in options:
            options['blocks'] = [{"type": "section", "text": {"type": "mrkdwn", "text": self.message}}]
        return options

    class Meta:
        verbose_name = 'Notification'
        verbose_name_plural = 'Notifications'
        indexes = [
            models.Index(fields=['status']),
        ]



from dataclasses import dataclass
from typing import List, Optional
from templated_email import get_templated_mail


class Block:
    def render_slack(self, **_):
        raise NotImplementedError('abstract method')

    def render_mail(self, **_):
        raise NotImplementedError('abstract method')


class Empty(Block):
    def render_slack(self, **_):
        return '', {}

    def render_mail(self, **_):
        return '', {}


class Message(Block, list[Block]):
    """
    list of blocks
    """

    def _render(self, block_met, **kw):
        message = []
        kwargs = {}
        for b in self:
            m, k = getattr(b, block_met)(**kw)
            message.append(m)
            full_merge_dict(kwargs, k)
        return '\n'.join(message), kwargs

    def render_slack(self, **_):
        return self._render('render_slack', **_)

    def render_mail(self, **_):
        return self._render('render_mail', **_)


@dataclass
class Basic(Block):
    """
    simple (plain) text message
    """

    message: str

    def render_slack(self, **_):
        return self.message, {}

    def render_mail(self, from_email=None, **_):
        # persist from_email in options
        options = {}
        if from_email:
            options['from_email'] = from_email
        return self.message, options


@dataclass
class Section(Basic):
    """
    block for "Section" type in slack. For mail, renders like Basic
    """

    def render_slack(self, **_):
        return self.message, {'blocks': [{'type': 'section', 'text': {'type': 'mrkdwn', 'text': f'{self.message}'}}]}


class Context(Block, list[str]):
    """
    block for "Context" type in slack. For mail, renders each element like Basic

    TODO: only supports str elements for now
    """

    @property
    def message(self):
        return '\n'.join(self)

    def render_slack(self, **_):
        return self.message, {
            'blocks': [{"type": "context", "elements": [{"type": "mrkdwn", "text": el} for el in self]}]
        }

    def render_mail(self, **_):
        return self.message, {}


@dataclass
class BasicSubject(Basic):
    subject: str

    def render_slack(self, **_):
        return f'{self.subject}: {self.message}', {}

    def render_mail(self, **_):
        m, o = super().render_mail(**_)
        o['subject'] = self.subject
        return m, o


@dataclass
class TemplatedMail(Basic):
    """
    render basic message for slack but use templated-email for email
    """

    template: str
    context: Optional[dict] = None
    create_link: Optional[bool] = False

    def render_mail(self, **kw):
        _, o = super().render_mail(**kw)
        template_message = get_templated_mail(
            template_name=self.template,
            context=self.context,
            from_email=kw.get('from_email'),
            create_link=self.create_link,
            to=kw.get('recipient_list'),
        )
        o['subject'] = template_message.subject
        for alt in template_message.alternatives:
            if alt[1] == 'text/html':
                o['html_message'] = alt[0]
                break

        return template_message.body, o


@dataclass
class ExtraRecipients(Empty):
    emails: List[str]

    def render_mail(self, **_):
        return self.message, {'recipient_list': self.emails}


def full_merge_dict(d_to, d_src):
    """
    based on https://stackoverflow.com/a/56042166

    * replace basic values
    * concatenate lists
    * update dictionaries
    """
    for key, value in d_to.items():
        if key in d_src:
            value2 = d_src[key]
            if type(value) != type(value2):
                raise ValueError('cannot merge different types', type(value), type(value2))
            if type(value) is dict:
                full_merge_dict(d_to[key], d_src[key])
            elif type(value) in (int, float, str):
                d_to[key] = d_src[key]
            elif type(value) is list:
                d_to[key].extend(d_src[key])
            else:
                raise ValueError('merge strategy unknown', type(value))
    for key, value in d_src.items():
        if key not in d_to:
            d_to[key] = value



# Generated by Django 3.1.5 on 2021-09-13 10:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('notifications', '0002_auto_20210913_1046'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='service',
            field=models.CharField(choices=[('S', 'Slack'), ('M', 'Mail')], max_length=1),
        ),
    ]



# Generated by Django 3.1.5 on 2021-09-13 10:46

from django.db import migrations


def data_fix(apps, schema_editor):
    apps.get_model('notifications', 'Subscription').objects.filter(service='T').update(service='M')


class Migration(migrations.Migration):
    dependencies = [
        ('notifications', '0001_initial_20210729'),
    ]

    operations = [
        migrations.RunPython(data_fix, reverse_code=data_fix),
    ]



from django.core.management.base import BaseCommand

from notifications import utils


class Command(BaseCommand):
    help = 'Test notifications'

    def add_arguments(self, parser):
        parser.add_argument('event_name', type=str, help='Event name')
        parser.add_argument('message', type=str, help='Message')
        parser.add_argument('-s', '--subject', type=str, default=None, help='Subject')
        parser.add_argument('--create-link', action='store_true', help='Create link')

    def handle(self, *args, **options):
        x = utils.notify(
            options['event_name'], options['message'], subject=options['subject'], create_link=options['create_link']
        )
        if not x:
            self.stderr.write('No notifications created... Is the event setup? Are there any subscriptions?')
        else:
            self.stdout.write(f'{x} notifications created')



import json
import logging

from django.conf import settings
from django.core.mail import EmailMultiAlternatives, get_connection, message
from templated_email import get_templated_mail
from typing import NamedTuple, Optional

from notifications.models import Event, Notification, Subscription
from . import blocks

logger = logging.getLogger(__name__)


class Attachment(NamedTuple):
    file_name: str
    file_path: str  # Only accepts files from MEDIA_ROOT
    file_type: str


def send_event_email(event, subject, message, recipient_list, html_message=None):
    with get_connection() as connection:
        msg = EmailMultiAlternatives(
            subject=subject,
            body=message,
            from_email=event.mail_from or settings.NOTIFICATIONS_MAIL_FROM,
            to=recipient_list,
            reply_to=[event.mail_reply_to] if event.mail_reply_to else None,  # None by default
            connection=connection,
        )
        if html_message:
            msg.attach_alternative(html_message, 'text/html')
        return bool(msg.send())


def notify_templated(event_name, template, context, **kwargs):
    return notify(
        event_name,
        None,
        template=template,
        context=context,
        **kwargs,
    )


def __notify_blocks(event_name, block, queryset=None):
    """
    ALPHA method to experiment with block building to simplify all the extra options
    check blocks.py for the supported blocks!
    """
    if event_name is not None:
        event = Event.objects.get(name=event_name)
        if queryset is None:
            queryset = event.subscription_set
    elif queryset:
        # enabled flag does not really matter here
        event = queryset.first().event
    else:
        return 0

    count = 0

    queryset = queryset.filter(enabled=True)
    targets = queryset.filter(service=Subscription.Service.SLACK)
    if targets:
        api_kwargs = event.slack_api_kwargs()
        message, extra_kwargs = block.render_slack()
        api_kwargs.update(extra_kwargs)
        for subscription in targets:
            for target in subscription.target.split('\n'):
                Notification.objects.create(
                    subscription=subscription, message=message, target=target.strip(), options=json.dumps(api_kwargs)
                )
                count += 1

    targets = queryset.filter(service=Subscription.Service.MAIL)
    if targets:
        try:
            recipient_list = {
                mail.strip() for target in targets.values_list('target', flat=True) for mail in target.split('\n')
            }
            mail_body, options = block.render_mail(
                from_email=event.mail_from or settings.NOTIFICATIONS_MAIL_FROM,
                recipient_list=recipient_list,
            )
            recipient_list.update(set(options.get('recipient_list', [])))
            recipient_list = list(recipient_list)
            options['reply_to'] = [event.mail_reply_to] if event.mail_reply_to else None

            # FIXME: add html_message and attachments!

            for target in targets:
                Notification.objects.create(
                    subscription=target,
                    target=json.dumps(recipient_list),
                    message=mail_body,
                    options=json.dumps(options),
                    status=Notification.STATUS_PENDING,
                )
        except Exception:
            logger.exception('error notifying %s', event.name)
        count += targets.count()

    return count


def notify(
    event_name,
    message,
    subject=None,
    html_message=None,
    queryset=None,
    template=None,
    context=None,
    create_link=False,
    additional_email_targets=None,
    attachments: Optional[list[Attachment]] = None,
    slack_attachments=None,
) -> int:
    if isinstance(message, blocks.Block):
        # temporarily support both calls (eventually deprecate non-blocks and this method)
        return __notify_blocks(event_name, message)

    count = 0

    if event_name is not None:
        event = Event.objects.get(name=event_name)
        if queryset is None:
            queryset = event.subscription_set
    elif queryset:
        # enabled flag does not really matter here
        event = queryset.first().event
    else:
        return 0

    queryset = queryset.filter(enabled=True)

    slack_text = f'{subject}: {message}' if subject else message
    api_kwargs = event.slack_api_kwargs()
    if slack_attachments:
        # TODO: can this be taken from a more "generic" arg and also use it in email?
        api_kwargs['attachments'] = slack_attachments
    for subscription in queryset.filter(service=Subscription.Service.SLACK):
        for target in subscription.target.split('\n'):
            Notification.objects.create(
                subscription=subscription, message=slack_text, target=target.strip(), options=json.dumps(api_kwargs)
            )
            count += 1

    targets = queryset.filter(service=Subscription.Service.MAIL)
    if targets:
        try:
            recipient_list = {
                mail.strip() for target in targets.values_list('target', flat=True) for mail in target.split('\n')
            }

            if additional_email_targets:
                recipient_list.update(set(additional_email_targets))

            recipient_list = list(recipient_list)

            mail_body = message
            mail_options = dict(
                subject=subject,
                from_email=event.mail_from or settings.NOTIFICATIONS_MAIL_FROM,
                reply_to=[event.mail_reply_to] if event.mail_reply_to else None,
                html_message=html_message,
                attachments=attachments,
            )

            prepare_and_store_notifications(
                template=template,
                event=event,
                create_link=create_link,
                recipient_list=recipient_list,
                context=context,
                mail_options=mail_options,
                targets=targets,
                mail_body=mail_body,
            )
        except Exception:
            logger.exception('error notifying %s', event.name)
        count += targets.count()

    return count


def prepare_and_store_notifications(
    template: str,
    event: Event,
    create_link: bool,
    recipient_list: list,
    context: str,
    mail_options: dict,
    targets: list,
    mail_body: str,
) -> None:
    if template:
        template_message = get_templated_mail(
            template_name=template,
            context=context,
            from_email=event.mail_from or settings.NOTIFICATIONS_MAIL_FROM,
            create_link=create_link,
            to=recipient_list,
        )
        mail_options["subject"] = template_message.subject
        mail_body = template_message.body
        for alt in template_message.alternatives:
            if alt[1] == 'text/html':
                mail_options["html_message"] = alt[0]
                break
    for target in targets:
        Notification.objects.create(
            subscription=target,
            target=json.dumps(recipient_list),
            message=mail_body,
            options=json.dumps(mail_options),
            status=Notification.STATUS_PENDING,
        )



import json

from django.contrib import admin
from django.contrib.admin.widgets import AdminTextInputWidget
from django import forms
from django.conf import settings
from django.contrib.admin.utils import unquote
from django.template.defaultfilters import truncatechars
from django.template.response import TemplateResponse
from django.http.response import HttpResponseNotFound
from django.core.exceptions import PermissionDenied
from django.utils.html import format_html_join
from django.utils.text import capfirst

from notifications import models, utils


class RandomTokenWidget(AdminTextInputWidget):
    template_name = 'django/forms/widgets/randomgeneratetext.html'

    class Media:
        js = ('django/forms/widgets/randomgeneratetext.js',)


@admin.register(models.Event)
class EventAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {'fields': ('name', 'external_token')}),
        ('Slack only', {'fields': ('slack_username', 'slack_icon', 'slack_unfurl_links')}),
        ('Mail only', {'fields': ('mail_from', 'mail_reply_to')}),
    )

    def formfield_for_dbfield(self, db_field, request, **kwargs):
        ff = super().formfield_for_dbfield(db_field, request, **kwargs)
        if db_field.name == 'external_token':
            ff.widget = RandomTokenWidget()
        return ff


class SubscriptionAdminForm(forms.ModelForm):
    def __validate_email_target(self):
        for x in self.cleaned_data.get('target', '').splitlines():
            if x and '@' not in x:
                self.add_error('target', f'{x} is not a valid email.')

    def __validate_slack_target(self):
        # faster cold boot
        import slack_sdk

        slackclient = slack_sdk.WebClient(token=settings.NOTIFICATIONS_SLACK_APP_TOKEN)
        # API only allows query user/channel by ID
        # using list to be able to validate on names will return near 10k records on each of the endpoints
        # easiest (and most accurate): post test message and check error message
        old_targets = set(self.initial.get('target', '').splitlines())
        new_targets = set(self.cleaned_data.get('target', '').splitlines())
        to_check = new_targets - old_targets
        for x in to_check:
            try:
                slackclient.chat_postMessage(
                    channel=x,
                    text=f':mega:  This channel just subscribed event *{self.cleaned_data.get("event")}* :newspaper:',
                    as_user=1,
                )
            except slack_sdk.errors.SlackApiError as e:
                self.add_error(
                    'target', f'{x} is not a valid slack channel/user (or it is private) - {e.response.data["error"]}'
                )

    def clean(self):
        if 'target' in self.changed_data:
            _s = self.cleaned_data.get('service')
            if _s == models.Subscription.Service.MAIL:
                self.__validate_email_target()
            elif _s == models.Subscription.Service.SLACK:
                self.__validate_slack_target()
        return self.cleaned_data


@admin.register(models.Subscription)
class SubscriptionAdmin(admin.ModelAdmin):
    form = SubscriptionAdminForm
    list_display = ('event', 'service', 'target', 'enabled')
    list_filter = ('event', 'service', 'target', 'enabled')
    search_fields = ('event__name', 'target')
    list_select_related = ('event',)
    actions = ['test_notification']

    def test_notification(self, request, queryset):
        count = utils.notify(None, 'Test notification', queryset=queryset)
        self.message_user(request, '%d notifications created' % count)

    test_notification.short_description = 'Send test notification'

    def get_actions(self, request):
        actions = super().get_actions(request)
        if not request.user.is_superuser and 'test_notification' in actions:
            del actions['test_notification']
        return actions


@admin.register(models.Notification)
class NotificationAdmin(admin.ModelAdmin):
    list_display = ('id', 'subscription', 'time', 'status', 'get_target')
    list_filter = (
        'time',
        'subscription',
        'status',
        'subscription__event',
        'subscription__service',
    )
    list_filter_select_related = {'subscription': ('event',)}
    readonly_fields = ('time', 'subscription', 'status', 'message', 'target', 'options')
    list_select_related = ('subscription', 'subscription__event')
    no_global_search = True

    def get_urls(self):
        from django.urls import path

        urls = super().get_urls()
        urls.insert(
            0,
            path(
                '<path:object_id>/preview/',
                self.admin_site.admin_view(self.preview_view),
                name='notifications_notification_preview',
            ),
        )
        return urls

    def preview_view(self, request, object_id, extra_context=None):
        # ref: ModelAdmin.history_view
        # First check if the user can view this notification.
        model = self.model
        obj = self.get_object(request, unquote(object_id))
        if obj is None:
            return self._get_obj_does_not_exist_redirect(request, model._meta, object_id)
        if not self.has_view_permission(request):
            raise PermissionDenied

        opts = model._meta
        context = {
            **self.admin_site.each_context(request),
            'title': 'Preview: %s' % obj,
            'module_name': str(capfirst(opts.verbose_name_plural)),
            'object_id': object_id,
            'original': obj,
            'object': obj,
            'message': obj.message,
            'opts': opts,
            'preserved_filters': self.get_preserved_filters(request),
            'has_view_permission': True,
            **(extra_context or {}),
        }
        request.current_app = self.admin_site.name

        if obj.subscription.service == models.Subscription.Service.SLACK:
            # block-builder preview limit is 3000 per full payload
            # TODO: improve this truncation in the future (for multiple blocks / attachments)
            # for now, truncate "text" to 1000..
            extra_options = obj.slack_options
            message_blocks = extra_options['blocks']
            message_blocks[0]['text']['text'] = truncatechars(message_blocks[0]['text']['text'], 1000)
            message_blocks = json.dumps({'blocks': message_blocks})
            attachment_blocks = [
                (json.dumps(_a['blocks'], indent=4), json.dumps(_a))
                for _a in extra_options.get('attachments', [])
                if 'blocks' in _a
            ]
            context.update(
                {
                    'message_blocks': message_blocks,
                    'slack_team': settings.NOTIFICATIONS_SLACK_TEAM or '',
                    'attachments': attachment_blocks,
                }
            )
            return TemplateResponse(request, "admin/notifications/notification/preview_slack.html", context)
        elif obj.subscription.service == models.Subscription.Service.MAIL:
            context['html_message'] = obj.options_dict.get('html_message')
            return TemplateResponse(request, "admin/notifications/notification/preview_mail.html", context)
        else:
            raise HttpResponseNotFound(f'{obj.subscription.service} not supported')

    def get_target(self, obj):
        targets = [obj.target]
        if obj.target and obj.target[:2] == '["':
            try:
                targets = json.loads(obj.target)
            except Exception:
                # let it be treated as single target
                pass
        return format_html_join('', '<span class="badge">{}</span>', ((target,) for target in targets))

    get_target.short_description = 'Target'
    get_target.admin_order_field = 'target'

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



# Generated by Django 3.1.5 on 2021-07-29 08:53

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    replaces = [
        ('notifications', '0001_initial_20210326'),
        ('notifications', '0003_auto_20200727_1101'),
        ('notifications', '0001_initial_v2'),
        ('notifications', '0001_squashed_0004_subscription_enabled'),
        ('notifications', '0001_initial'),
        ('notifications', '0002_auto_20180920_1631'),
        ('notifications', '0003_unfurl'),
        ('notifications', '0004_subscription_enabled'),
        ('notifications', '0002_auto_20191024_0917'),
    ]

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('name', models.CharField(max_length=50, primary_key=True, serialize=False)),
                (
                    'external_token',
                    models.CharField(
                        blank=True,
                        help_text='If set, notifications be posted to this event through the API',
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    'slack_username',
                    models.CharField(
                        blank=True,
                        help_text='Choose a display name for Slack bot (instead of default)',
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    'slack_icon',
                    models.CharField(
                        blank=True,
                        help_text='Choose an icon for Slack bot (instead of default)',
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    'slack_unfurl_links',
                    models.BooleanField(
                        default=True, help_text='Pass true to enable unfurling of primarily text-based content on Slack'
                    ),
                ),
                (
                    'mail_from',
                    models.CharField(
                        blank=True,
                        help_text='Choose the sender address of the email (instead of default)',
                        max_length=200,
                        null=True,
                    ),
                ),
                (
                    'mail_reply_to',
                    models.CharField(
                        blank=True,
                        help_text='Choose the reply-to address of the email (instead of none)',
                        max_length=200,
                        null=True,
                    ),
                ),
            ],
            options={
                'verbose_name': 'Event',
                'verbose_name_plural': 'Events',
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'service',
                    models.CharField(choices=[('S', 'Slack'), ('M', 'Mail'), ('T', 'Mail TLA Owners')], max_length=1),
                ),
                ('target', models.TextField(blank=True, help_text='You can specify multiple targets, one per line.')),
                ('enabled', models.BooleanField(default=True)),
                (
                    'event',
                    models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='notifications.event'),
                ),
            ],
            options={
                'verbose_name': 'Subscription',
                'verbose_name_plural': 'Subscriptions',
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time', models.DateTimeField(auto_now_add=True)),
                ('message', models.TextField()),
                ('status', models.IntegerField(choices=[(0, 'Pending'), (1, 'Sent'), (-1, 'Error')], default=0)),
                ('target', models.TextField(default=None, null=True)),
                ('options', models.TextField(default=None, null=True)),
                (
                    'subscription',
                    models.ForeignKey(
                        null=True, on_delete=django.db.models.deletion.SET_NULL, to='notifications.subscription'
                    ),
                ),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
            },
        ),
    ]



__version__ = '0.0.5'

# set default_app_config when using django earlier than 3.2
try:
    import django

    if django.VERSION < (3, 2):
        default_app_config = 'notifications.apps.NotificationsConfig'
except ImportError:
    pass



from django.apps import AppConfig
from django.conf import settings

# IMPORT do not use "app_settings.py strategy" as that is not compatible with @override_settings (unittests)
# this strategy is
APP_SETTINGS = dict(
    MAIL_FROM=None,
    SLACK_APP_TOKEN=None,
    SLACK_TEAM=None,
)


class NotificationsConfig(AppConfig):
    name = 'notifications'
    default_auto_field = 'django.db.models.AutoField'

    def ready(self):
        for k, v in APP_SETTINGS.items():
            _k = 'NOTIFICATIONS_%s' % k
            if not hasattr(settings, _k):
                setattr(settings, _k, v)



from django.http.response import JsonResponse
from django.views.decorators.csrf import csrf_exempt

from notifications import models, utils


@csrf_exempt
def notify(request):
    if request.method != 'POST':
        return JsonResponse({'error': 'invalid method'}, status=400)

    event = request.POST.get('event')
    if event is None:
        return JsonResponse({'error': 'event not found'}, status=404)
    try:
        ev = models.Event.objects.get(name=event)
    except models.Event.DoesNotExist:
        return JsonResponse({'error': 'event not found'}, status=404)

    if not ev.external_token or ev.external_token != request.POST.get('token'):
        return JsonResponse({'error': 'invalid token'}, status=403)

    if not request.POST.get('message'):
        return JsonResponse({'error': 'missing message'}, status=400)

    c = utils.notify(
        ev.name,
        request.POST.get('message'),
        subject=request.POST.get('subject'),
        html_message=request.POST.get('html_message'),
    )

    return JsonResponse({'notifications': c}, status=200)
